package com.wtkj.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.entity.AuthUserInfo;
import com.wtkj.entity.DeptToDept;
import com.wtkj.service.IDeptToDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.tool.api.R;
import org.springblade.system.entity.Dept;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-09-14 11:13
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/common")
@Api(value = "公共模块", tags = "公共模块")
public class CommonController {

	private final IDeptToDeptService deptToDeptService;

	/**
	 * 当前登录用户信息
	 */
	@GetMapping("/user-info")
	@ApiOperation(value = "当前登录用户信息", notes = "")
	@ApiOperationSupport(order = 1)
	public R<AuthUserInfo> userInfo() {
		return R.data(deptToDeptService.userInfo());
	}

	/**
	 * 新建机构
	 */
	@PostMapping("/sava-dept")
	@ApiOperation(value = "新建政府机构", notes = "传入DeptToDept")
	@ApiOperationSupport(order = 2)
	public R<Boolean> createDept(@RequestBody @Valid DeptToDept dept) {
		return R.status(deptToDeptService.saveOrUpdate(dept));
	}

	/**
	 * 机构搜索
	 */
	@GetMapping("/get-dept")
	@ApiOperation(value = "当前登录用户信息", notes = "")
	@ApiOperationSupport(order = 3)
	public R<List<Dept>> list(Long topDept, String areaCode, String name, Integer category) {
		return R.data(deptToDeptService.getDept(topDept, areaCode, name, category));
	}
}
